// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      DopeVector.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 16:47:44 MEST 2002

#ifndef DopeVector_h
#define DopeVector_h
#include <f90cxx.h>



/************************************************************
*  Templated interface to F90 vectors of a particular type.
*
*************************************************************/

#ifdef NAGf90Fortran
template<typename T, int N = 1>
struct DopeVector
{
public:


    DopeVector() : addr(0) { }

    DopeVector(const T* vec, const unsigned long* ext, const unsigned long* low=0) 
    {
        addr = const_cast<T*>(vec);		// address to first element
        offset = 0;
      

        // Swap C++ and Fortran indices
        for (int i = N-1, j = 0; i >= 0; --i, ++j) 
        {
            dim[j].extent	= ext[i];
            dim[j].lower	= (low ? low[i] : 0);

            if (j == 0)
                dim[j].mult = sizeof(T);
            else
                dim[j].mult = dim[j-1].mult * dim[j-1].extent;

            offset -= dim[j].mult * dim[j].lower;
        }
       

    }    
  
    DopeVector(const T* vec, const std::vector<unsigned long>& ext,
	    const std::vector<unsigned long>& low=std::vector<unsigned long>( 0) ) 
    {
      

        assert(ext.size() == N);
        assert(low.size() == 0 || low.size() == N);

        addr = const_cast<T*>(vec);		// address to first element
        offset = 0;

        // Swap C++ and Fortran indices
        for (int i = N-1, j = 0; i >= 0; --i, ++j) 
        {
            dim[j].extent	= ext[i];
            dim[j].lower	= (low.size() > 0 ? low[i] : 0);

            if (j == 0)
                dim[j].mult = sizeof(T);
            else
                dim[j].mult = dim[j-1].mult * dim[j-1].extent;

            offset -= dim[j].mult * dim[j].lower;
        }

    

    }

    T* addr;
  

//    Integer3 offset;
    dope_Offset offset;
    Triplet dim[1];

    int length() const
    {
     

        return dim[0].extent;
    }

    int lbound() const
    {
    


        return dim[0].lower;
    }

    int ubound() const
    {


   

        return dim[0].lower+dim[0].extent-1;
    }

    T& operator[]( unsigned long idx ) const
    {
 

      
// substituted int by dope_Offset type, which is either long long (64-bit) or int (32).

        return *(T*)((dope_Offset)addr + (dim[0].mult*idx + offset));
    }

    int sizeOfType() const {
   


        return sizeof(T);
    }
#else
template<typename T, int N = 1>
struct DopeVector
{
public:

DopeVector() : addr(0) { }

DopeVector(const T* vec, const unsigned long* ext, const unsigned long* low=0)
{
	addr = const_cast<T*>(vec);		// address to first element
   offset = 0;
     if (low != 0)
       {
         if (*low == 1)
           {
             span =  1;
             low = 0;
           }
         else if ( *low == 2)
           {
             span = 1;
             low = 0;
           }
         else if ( (*low > 2) && (*low < 8) )
           {
             span = 4;
             low = 0;
           }
         else if ((*low == 8 ) )
           {
             span = 8;
             low = 0;
           }
         else
           {
             span = 1;
             low = 0;
           }
       }

    dtype.rank = N;
    dtype.elem_len = 0;
    dtype.type = 1;

    // Swap C++ and Fortran indices
    for (int i = N-1, j = 0; i >= 0; --i, ++j) {
      dim[j]._ubound    =  ext[i]-1;
      dim[j].lower_bound        = (low ? low[i] : 0);

      if (j == 0)
        dim[j]._stride =  1;
      else      
          dim[j]._stride =  dim[j-1]._stride * (dim[j-1]._ubound + 1 -  dim[j-1].lower_bound) ;

      offset -= dim[j]._stride * dim[j].lower_bound;
    } 
}

DopeVector(const T* vec, const std::vector<unsigned long>& ext,
		const std::vector<unsigned long>& low=std::vector<unsigned long>( 0) )
{


	assert(ext.size() == N);
	assert(low.size() == 0 || low.size() == N);

	addr = const_cast<T*>(vec);		// address to first element
    offset = 0;
    dtype.rank = N;
    dtype.elem_len = 0;
    dtype.type = 0;

    // Swap C++ and Fortran indices
    for (int i = N-1, j = 0; i >= 0; --i, ++j) {
      dim[j]._ubound    = ext[i]-1;
      dim[j].lower_bound        = 0;

      if (j == 0)
        dim[j]._stride = 1;
      else
          dim[j]._stride =  dim[j-1]._stride * (dim[j-1]._ubound + 1 -  dim[j-1].lower_bound) ;

      offset -= dim[j]._stride * dim[j].lower_bound;
    }
}

  T* addr;
  size_t offset;
  dtype_type dtype;
  index_type span;
  descriptor_dimension dim[N];


int length() const
{
	return dim[0]._ubound +1 - dim[0].lower_bound;
}

int lbound() const
{
	return dim[0].lower_bound;
}

int ubound() const
{
	return dim[0]._ubound;
}

T& operator[]( unsigned long idx ) const
{
	// substituted int by dope_Offset type, which is either long long (64-bit) or int (32).
        // not longer valid since gcc8
	//return *(T*)((long long)addr + (dim[0]._stride*idx + offset));
        // New array descriptor definition
        return *(T*)((long long)addr + ((dim[0].lower_bound*dim[0]._stride*idx) + offset)*span);
}

int sizeOfType() const {

	return sizeof(T);
}
#endif

private:


};

#endif
