// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/****************************************************************************
*
*
* Dialogue for doing interactive photometry on FAST mode images
*
****************************************************************************/

// QT header files


#include <qapplication.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qcombobox.h>
#include <qwhatsthis.h>
#include <qinputdialog.h>
#include <qtimer.h>
#include <cmath>
#include<cstring>
#include <qstring.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h> 
#include <qpaintdevicemetrics.h> 
#include <qdatetime.h>
#include <qmenubar.h>
#include <qmessagebox.h>

// omsource header files

#include "OmSourceLog.h"
#include "OmImageDisplay.h"
#include "OmSourceRegions.h"
#include "OmSourceVersionNo.h"
#include "OmProgressDialog.h"
#include "OmFastPhotometryDlg.h"
#include "OmSourceNumerics.h"
#include "OmSourceProcessDlg.h"
#include "OmSourceInformationDlg.h"
#include "OmSourceBackgroundDlg.h"

//#include "OmSourceGraphDlg.h"
#include "OmSourceBrowser.h"
#include "OmSourceButton.h"
#include "OmSourceGroupBox.h"
#include "OmSourceLabel.h"
#include "OmSourceInformationMenu.h"
#include "OmSourceHelp.h" 
#include "OmSourceEdit.h"
#include "OmSourceRegions.h"
#include "OmSourceLabelEdit.h"
#include"OmSpinBox.h"

//#include "graph.h"
/* 
 *  Constructs a PhotometryDlg which is a child of 'parent', with the computeB  
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */

// FORTRAN prototypes
F90MODPROCPROTO(void,omsource_glue, recompute)(const int& id);
F90MODPROCPROTO(void, omsource_glue, replace)(const int &id, OmSourceParameter& source);
F90MODPROCPROTO(void, omsource_glue, getbackgroundregions)(OmSourceRegions *regions);
F90MODPROCPROTO(void, omsource_glue, getthecounts)(const float &radius);
F90MODPROCPROTO(void, omsource_glue, getsourcecounts)(const float &x, 
             const float &y, const float &radius, float &counts);
F90MODPROCPROTO(void, omsource_glue, getthebackgroundcounts)(const float &x, 
             const float &y, const float &radius, float &r1, float &r2, float &counts);
F90MODPROCPROTO(void, omsource_glue, computebackground)(float &counts, float &error);
F90MODPROCPROTO(void,omsource_glue, deletebackgroundlist)();
F90MODPROCPROTO(float, omsource_glue, getthebackgroundmode)();


/**********************************************************************************
 * For recomputing the photometry of a source
 **********************************************************************************/
F90MODPROCPROTO(void, omsource_glue, recompute_photometry)(int *numberToFit, int *no, 
                int *, double *, double *, double *, double*, double*, double*);



F90MODPROCPROTO(void, omsource_old, setweightingmethod)(int *method);

extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mybackgroundsources);
extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);


extern F90MODVARDEF(int, omsource_glue, myimagenx);
extern F90MODVARDEF(int, omsource_glue, myimageny);

extern F90MODVARDEF(int, omsource_glue, mynosources);




string FastPhotometryDlg::_colour1 = "blue", FastPhotometryDlg::_colour2 = "cyan";
int FastPhotometryDlg::_colour1No = 4, FastPhotometryDlg::_colour2No = 5;

const int noChars = 30;
/***************************************************************************
*
***************************************************************************/

bool compare(const OmSourceParameter  &a, const OmSourceParameter & b)
{
    return a.alpha < b.alpha;

};





 
/****************************************************
*
* Constructor
* Photometry dialogue for FAST mode data
*
*****************************************************/
FastPhotometryDlg::FastPhotometryDlg(const int no, 
			OmImageDisplay *displayIn, 
                        const OmSourceBrowser::ObsDetails* obsDetails,
                        const OmSourceBrowser::ImageDetails* imageDetails,
			const string oldRegionsColour, 
			vector<string> colourList,
                        QWidget* parent,  
                        const char* name, 
                        bool modal, 
                        WFlags fl)
  : QDialog( parent, name, modal, fl ), 
    _sourceNo(no),_photometryType(0), 
    _display(displayIn),  _obsDetails(obsDetails),  
    _imageDetails(imageDetails), _zoomFactor(0), _displayUpdated(false), 
    _fixedBackground(false), _oldRegionsColour(oldRegionsColour),
    _colourList( colourList )




{


    /***********************************************************************************
    * Need the image dimensions to stop the user selecting sources outside the image
    *********************************************************************************/

    _nx = (float)(F90MODPROC( omsource_glue, myimagenx ) + 1);
    _ny = (float)(F90MODPROC( omsource_glue, myimageny ) + 1);

    _noSources = F90MODPROC( omsource_glue, mynosources);




    //  cout << "Line 169 mynosources=" << _noSources << " recomputing photometry for source" << _sourceNo << endl;


    _timer = 0;
    //  QToolTip::add(this, "This is a test");
    string text2 = "Constructing photometry window- please wait";
   
    OmProgressDialog *reporter = new OmProgressDialog(text2, 41);
    
    reporter->setProgress(1);
   
     // load display
    //    _display = new OmImageDisplay;
     
    reporter->setProgress(2);
  
   
    if ( !name )
	setName( "Photometry" );
    //resize( 580, 950 ); 
    resize( 600, 950 ); 
  
    QString text;
    text.sprintf("Fast mode photometry (Source no %d, Filter=%s, exposure=%6d seconds)", no,  
      (const char*)_obsDetails->filter,  (int)_obsDetails->exposure);

    setCaption( tr( text) );

    int height = 25;
    int yPos = 25;

    /*******************************************************
    *
    * Raw count rates
    *
    ******************************************************/
    _rawRateGroupBox = new OmSourceGroupBox( this,  "Raw count rates (counts/sec)",  10, yPos, 230, 140,
          "The following rates are those obtained from the source photometry\n"
	  "before any corrections have been made");

    //    QToolTip::add(_rawRateGroupBox, "The following rates are those obtained from the source photometry\n"
    //	  "before any corrections have been made");



    reporter->setProgress(2);

   

    // Total
    int xPos = 130;
    xPos = 10;
    int pmSpacing = 2;
  

    _rawTotalRateTxt = new OmSourceLabel(_rawRateGroupBox, "Total", xPos, yPos, height,
         "(Backgound counts + source counts) / exposure (counts/sec)");
  
 
    xPos += 90;
    _rawTotalRateEdt = new OmSourceEdit(   _rawRateGroupBox, "400.0", xPos, yPos, 50, height); 
    reporter->setProgress(3);
   
  
    xPos = _rawTotalRateEdt->x() +  _rawTotalRateEdt->width() + pmSpacing;
 
    _rawTotalRateErrTxt = new OmSourceLabel(_rawRateGroupBox,  tr( "+/-" ),  xPos, yPos, height,
       "The error in the total rate (counts/sec)" );
    reporter->setProgress(4);
   
 
    xPos = _rawTotalRateErrTxt->x() +  _rawTotalRateErrTxt->width() + pmSpacing;
 
    _rawTotalRateErrEdt = new OmSourceEdit(   _rawRateGroupBox, "400.0", xPos, yPos, 40, height  ); 
   
  
    // Background
    xPos = 10;
    yPos += 40;
    _rawBackgroundRateTxt = new OmSourceLabel(_rawRateGroupBox, "Background", xPos, yPos, height,
       "Backgound counts / exposure (counts/sec)");
    
   
    xPos += 90;
    _rawBackgroundRateEdt = new OmSourceEdit(   _rawRateGroupBox, "0.00", xPos, yPos, 50, height ); 
    reporter->setProgress(5);
   

 
    xPos = _rawBackgroundRateEdt->x() +  _rawBackgroundRateEdt->width() + pmSpacing;
 
    _rawBackgroundRateErrTxt = new OmSourceLabel(_rawRateGroupBox, tr( "+/-" ),  xPos, yPos, height);
    reporter->setProgress(7);
   
 
    xPos = _rawBackgroundRateErrTxt->x() +  _rawBackgroundRateErrTxt->width() + pmSpacing;
 
    _rawBackgroundRateErrEdt = new OmSourceEdit( _rawRateGroupBox, "100.0", xPos, yPos, 40, height ); 
  
 
    // Background subtracted

    xPos = 10;
    yPos += 40;
    _rawBackgroundSubtractedRateTxt = new OmSourceLabel(_rawRateGroupBox, "Source",  xPos, yPos, height,
                "Backgound-subtracted counts / exposure (counts/sec)");
      
    xPos += 90;
    _rawBackgroundSubtractedRateEdt = new OmSourceEdit(   _rawRateGroupBox, "4000.0", xPos, yPos, 50, height  ); 
    reporter->setProgress(6);
   

  


    xPos = _rawBackgroundSubtractedRateEdt->x() +  _rawBackgroundSubtractedRateEdt->width() + pmSpacing;
 
    _rawBackgroundSubtractedRateErrTxt = new OmSourceLabel(_rawRateGroupBox, tr( "+/-" ),  xPos, yPos, height);
    reporter->setProgress(7);
   
 
    xPos = _rawBackgroundSubtractedRateErrTxt->x() +  _rawBackgroundSubtractedRateErrTxt->width() + pmSpacing;
 
    _rawBackgroundSubtractedRateErrEdt = new OmSourceEdit( _rawRateGroupBox, "100.0", xPos, yPos, 40, height ); 
  
    reporter->setProgress(8);
   

    /****************************************************************************
     * Corrected count rates
     ***************************************************************************/
    yPos = _rawRateGroupBox->geometry().top();

    _corrRateGroupBox = new OmSourceGroupBox( this,  "Corrected count rates from the photometry area ", 
					      //280, yPos, 230, 160,
					      280, yPos, 250, 160,
         "The following rates are those obtained from the source photometry\n"
	  "area after the listed corrections have been made");
    //    QToolTip::add(_corrRateGroupBox, "The following rates are those obtained from the source photometry\n"
    //	  "after the listed corrections have been made");
 
    


    // Total
    xPos = 10;
    yPos = 20;
    _corrTotalRateTxt = new OmSourceLabel(_corrRateGroupBox, "Total", xPos, yPos, height,
          "Corrected (Backgound counts + source counts) / exposure (counts/sec)");
    
 
    xPos += 90;
    _corrTotalRateEdt = new OmSourceEdit(   _corrRateGroupBox, "400.0", xPos, yPos, 50, height ); 
    reporter->setProgress(9);
   
  
    xPos = _corrTotalRateEdt->x() +  _corrTotalRateEdt->width() + pmSpacing;
 
    _corrTotalRateErrTxt = new OmSourceLabel(_corrRateGroupBox,  tr( "+/-" ),  xPos, yPos, height,
      "The error in the corrected total rate (counts/sec)" );
  
    reporter->setProgress(10);
   
 
    xPos = _corrTotalRateErrTxt->x() +  _corrTotalRateErrTxt->width() + pmSpacing;
 
    _corrTotalRateErrEdt = new OmSourceEdit(   _corrRateGroupBox, "400.0", xPos, yPos, 40, height  ); 
    reporter->setProgress(11);

  
 
   
    /*******************************************  
    * Background
    *******************************************/
    xPos = 10;
    yPos += 40;
    _corrBackgroundRateTxt = new OmSourceLabel(_corrRateGroupBox, "Background", xPos, yPos, height,
         "Corrected backgound counts / exposure (counts/sec)");
    

    xPos += 90;
    _corrBackgroundRateEdt = new OmSourceEdit(_corrRateGroupBox, "4000.0", xPos, yPos, 50, height ); 
    reporter->setProgress(12);
   
    xPos = _corrBackgroundRateEdt->x() +  _corrBackgroundRateEdt->width() + pmSpacing; 
    _corrBackgroundRateErrTxt = new OmSourceLabel(_corrRateGroupBox, tr( "+/-" ),  xPos, yPos, height);
   
    xPos = _corrBackgroundRateErrTxt->x() +  _corrBackgroundRateErrTxt->width() + pmSpacing;
    _corrBackgroundRateErrEdt = new OmSourceEdit( _corrRateGroupBox, "100.0", xPos, yPos, 40, height ); 
  
  
 
    /****************************************
    * Background subtracted
    *****************************************/
    xPos = 10;
    yPos += 40;
    _corrBackgroundSubtractedRateTxt = new OmSourceLabel(_corrRateGroupBox, "Source",  xPos, yPos, height,
	   "The corrected background-subtracted counts / exposure (counst/sec)");
 
  
    xPos += 90;
    _corrBackgroundSubtractedRateEdt = new OmSourceEdit(   _corrRateGroupBox, "4000.0", xPos, yPos, 50, height  ); 
    reporter->setProgress(13);
    
  
    xPos = _corrBackgroundSubtractedRateEdt->x() +  _corrBackgroundSubtractedRateEdt->width() + pmSpacing;
 
    _corrBackgroundSubtractedRateErrTxt = new OmSourceLabel(_corrRateGroupBox, tr( "+/-" ),  xPos, yPos, height,
	    "The error in the corrected background-subtracted count-rate (counts/sec)");
   
    reporter->setProgress(14);
   
    xPos = _corrBackgroundSubtractedRateErrTxt->x() +  _corrBackgroundSubtractedRateErrTxt->width() + pmSpacing;
 
    _corrBackgroundSubtractedRateErrEdt = new OmSourceEdit(   _corrRateGroupBox, "100.0", xPos, yPos, 40, height ); 
    reporter->setProgress(15);
   
 

    /********************************************************
     * Corrections applied
     *******************************************************/
        
    _correctionsGroupBox = new OmSourceGroupBox( this,  "Corrections applied",  10, 170, 190, 500,
	  "The corrections that have been applied in converting from raw to corrected count-rates (listed in order)");

    //    QToolTip::add(_correctionsGroupBox,  "The corrections that have been applied in converting from raw to corrected rates");

    int maxWidth = 0; 

    xPos = 10;
    yPos = 20;
 
    _psf1CorrectionTxt = new OmSourceLabel(_correctionsGroupBox, "PSF1",  xPos, yPos, height,
      "The correction (expressed as a ratio) applied to extrapolate,\n"
      "using the point-spread function, the raw counts measured within the\n"
      "aperture radius to that within the coincidence-loss radius");
    QToolTip::add(_psf1CorrectionTxt, "The correction (expressed as a ratio) applied to extrapolate,\n"
      "using the point-spread function, the raw background-subtracted count rate  measured within the\n"
      "aperture radius to that within the coincidence-loss radius\n"
      "Please note that the raw-background count rate is extrapolated to the coincidence-loss\n"
		  "ratio by the multiplication factor (coincidence-loss radius / aperture radius)**2");

    maxWidth = max(maxWidth, _psf1CorrectionTxt->geometry().right() );


  



    // Total
    int dy = 35;


    yPos += dy;
     _coincCorrectionTxt1 = new OmSourceLabel(_correctionsGroupBox,  tr( "B Coinc-loss"), xPos, yPos, height,
	   "The correction applied (expressed as a ratio) to correct the background count-rate for coincidence-losses\n"
	  "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");
  
    QToolTip::add(_coincCorrectionTxt1,  
          "The correction applied (expressed as a ratio) to correct the background count-rate for coincidence-losses\n"
	  "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");

    maxWidth = max(maxWidth, _coincCorrectionTxt1->geometry().right() );

 

    xPos = 10;
    yPos += dy;
    _coincCorrectionTxt2 = new OmSourceLabel(_correctionsGroupBox,  tr( "S+B Coinc-loss"), xPos, yPos, height,
	   "The correction applied (expressed as a ratio) to correct the source+background count-rate for coincidence-losses\n"
	   "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");

     QToolTip::add(_coincCorrectionTxt2,  
          "The correction applied (expressed as a ratio) to correct the source+background count-rate  for coincidence-losses\n"
  	  "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");

    maxWidth = max(maxWidth, _coincCorrectionTxt2->geometry().right() );


 

   
 
 
    reporter->setProgress(16);
     
  
    xPos = 10;
    yPos += dy;
    _psf2CorrectionTxt = new OmSourceLabel(_correctionsGroupBox, "PSF2",  xPos, yPos, height,
            "A correction (expressed as a percentage), only applicable to UV data,\n"
            " to extrapolate, using the point-spread function, the counts to the\n"
            "area of the UV point-spread function"); 
    QToolTip::add(_psf2CorrectionTxt,    "A correction (expressed as a ratio),\n"
         " to extrapolate, using the point-spread function, the counts to the\n"
         "area of the UV point-spread function\n"
		  "NB This is only applicable to UV data"); 

    maxWidth = max(maxWidth, _psf2CorrectionTxt->geometry().right() );


  

    reporter->setProgress(17);
   
 

    xPos = 10;
    yPos += dy;
    _tdsCorrectionTxt = new OmSourceLabel(_correctionsGroupBox, "TDS corr", xPos, yPos, height,
              "The time-dependent sensitivity correction applied");
    QToolTip::add(_tdsCorrectionTxt, "The time-dependent sensitivity correction applied");
 

    maxWidth = max(maxWidth, _tdsCorrectionTxt->geometry().right() );


    //********************************************************
    // The edit boxes to display the actual values
    //********************************************************

    xPos = maxWidth + 20;
    maxWidth = 0;

    yPos = _psf1CorrectionTxt->geometry().top();





 
   
 


    _psf1CorrectionEdt = new OmSourceEdit(   _correctionsGroupBox, "1.000", xPos, yPos, 50, height ); 
   

    QToolTip::add(_psf1CorrectionEdt, "The correction (expressed as a ratio) applied to extrapolate,\n"
      "using the point-spread function, the raw background-subtracted count rate  measured within the\n"
      "aperture radius to that within the coincidence-loss radius\n"
      "Please note that the raw-background count rate is extrapolated to the coincidence-loss\n"
		  "ratio by the multiplication factor (coincidence-loss radius / aperture radius)**2");







    maxWidth = max(maxWidth, _psf1CorrectionEdt->geometry().right());


   yPos += dy;

 


   _coincCorrectionEdt1 = new OmSourceEdit(   _correctionsGroupBox, "4000.0", xPos, yPos, 50, height  ); 
     
    QToolTip::add(_coincCorrectionEdt1,  
          "The correction applied (expressed as a ratio) to correct the background count-rate for coincidence-losses\n"
	  "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");


 


    yPos += dy;


   _coincCorrectionEdt2 = new OmSourceEdit(   _correctionsGroupBox, "4000.0", xPos, yPos, 50, height  ); 
   
     QToolTip::add(_coincCorrectionEdt2,  
          "The correction applied (expressed as a ratio) to correct the source+background count-rate  for coincidence-losses\n"
  	  "PLEASE NOTE THAT THIS ALSO INCLUDES the DEAD-TIME CORRECTION");

    maxWidth = max(maxWidth, _coincCorrectionEdt2->geometry().right());

 


    yPos += dy;
    _psf2CorrectionEdt = new OmSourceEdit(   _correctionsGroupBox, "1.000", xPos, yPos, 50, height ); 
    QToolTip::add(_psf2CorrectionEdt,    "A correction (expressed as a ratio) to extrapolate,\n"
          "using the point-spread function, the background-subtracted count-rate to the area of the UV point-spread function\n"
		  "NB This is only applicable to UV data");
          
    maxWidth = max(maxWidth, _psf2CorrectionEdt->geometry().right());


    yPos += dy;

    _tdsCorrectionEdt = new OmSourceEdit(  _correctionsGroupBox, "0.23478", xPos, yPos, 50, height );
    QToolTip::add(_tdsCorrectionEdt, "The time-dependent sensitivity correction applied");
   
    maxWidth = max(maxWidth, _tdsCorrectionEdt->geometry().right());



    _correctionsGroupBox->resize( maxWidth + 10, _tdsCorrectionEdt->geometry().bottom() + 20);



    reporter->setProgress(18);
   
  
 

    /*************************************************
     * Instrumental magnitudes, AB mags and fluxes
     ************************************************/
     yPos = _corrRateGroupBox->geometry().bottom() + 10;
    _magnitudeGroupBox = new OmSourceGroupBox( this,  "Instrumental magnitude/flux",  280, yPos, 230, 150,
			 "The magnitude obtained from the corrected count-rate");
   
    //    QToolTip::add(_magnitudeGroupBox, "The following have been computed using the corrected count-rates");


    xPos = 10;
    yPos = 20;
    _magnitudeTxt = new OmSourceLabel(_magnitudeGroupBox, "Magnitude",  xPos, yPos, height);
    QToolTip::add(_magnitudeTxt, "The instrumental magnitude");

 
    xPos += 90;
    _magnitudeEdt = new OmSourceEdit(   _magnitudeGroupBox, "40.9", xPos, yPos, 50, height); 
    QToolTip::add(_magnitudeEdt, "The instrumental magnitude");

    reporter->setProgress(19);
   
 
    xPos = _magnitudeEdt->x() +  _magnitudeEdt->width() + pmSpacing;
 
    _magnitudeErrTxt = new OmSourceLabel(_magnitudeGroupBox,  tr( "+/-" ),  xPos, yPos, height,
       "The error in the instrumental magnitude");

 
    xPos = _magnitudeErrTxt->x() +  _magnitudeErrTxt->width() + pmSpacing;
   
    _magnitudeErrEdt = new OmSourceEdit(   _magnitudeGroupBox, "0.13", xPos, yPos, 40, height); 
    reporter->setProgress(20);
   
 
    xPos = 10;
    yPos += 40;
    _signalToNoiseTxt = new OmSourceLabel(_magnitudeGroupBox, "Signal/noise",  xPos, yPos, height,
           "The ratio source counts / sqrt(source+background counts)\n"
	   "For point-sources this is computed using a 3 arcsec aperture");
    
 

    xPos += 90;
    _signalToNoiseEdt = new OmSourceEdit(   _magnitudeGroupBox, "5000.0", xPos, yPos, 50, height,
           "The ratio source counts / sqrt(source+background counts)\n" 
	   "For point-sources this is computed using a 3 arcsec aperture");
    


    /*****************************************************************************************
    * AB Magnitudes
    *****************************************************************************************/
    xPos = 10;
    yPos += 40;
    _ABMagnitudeTxt = new OmSourceLabel(_magnitudeGroupBox, "AB Magnitude",  xPos, yPos, height);
    QToolTip::add(_ABMagnitudeTxt, "The AB magnitude");

 
    xPos += 90;
    _ABMagnitudeEdt = new OmSourceEdit(   _magnitudeGroupBox, "40.9", xPos, yPos, 50, height); 
    QToolTip::add(_magnitudeEdt, "The AB magnitude");

    reporter->setProgress(19);
   
 
    xPos = _ABMagnitudeEdt->x() +  _ABMagnitudeEdt->width() + pmSpacing;
 
    _ABMagnitudeErrTxt = new OmSourceLabel(_magnitudeGroupBox,  tr( "+/-" ),  xPos, yPos, height,
       "The error in the AB magnitude");

 
    xPos = _ABMagnitudeErrTxt->x() +  _ABMagnitudeErrTxt->width() + pmSpacing;
   
    _ABMagnitudeErrEdt = new OmSourceEdit(   _magnitudeGroupBox, "0.13", xPos, yPos, 40, height); 
    reporter->setProgress(20);
   
 

    /*****************************************************************************************
    * AB Fluxes
    *****************************************************************************************/
    xPos = 10;
    yPos += 40;
    _ABFluxTxt = new OmSourceLabel(_magnitudeGroupBox, "AB Flux Density",  xPos, yPos, height);
    QToolTip::add(_ABMagnitudeTxt, "The AB flux");

 
    xPos = _ABMagnitudeTxt->geometry().right() + 20;
    _ABFluxEdt = new OmSourceEdit(   _magnitudeGroupBox, "0.000000000000001", xPos, yPos, 50, height); 
    QToolTip::add(_ABFluxEdt, "The AB flux density (ergs/cm**2/sec/A)");



    yPos = _ABFluxEdt->geometry().bottom();
    xPos = 10;
 
    _ABFluxErrTxt = new OmSourceLabel(_magnitudeGroupBox,  tr( "+/-" ),  xPos, yPos, height,
       "The error in the AB flux density (ergs/cm**2/sec/A)");

 
   
    xPos = _ABFluxEdt->geometry().left();
    _ABFluxErrEdt = new OmSourceEdit(   _magnitudeGroupBox, "0.000000000001", xPos, yPos, 50, height); 
    reporter->setProgress(20);
   
    _magnitudeGroupBox->resize( _magnitudeGroupBox->width(), _ABFluxErrEdt->geometry().bottom() + 10);




    reporter->setProgress(21);
   
 


    /***********************************************************
    * Photometry Method
    **********************************************************/ 
   
    xPos = 10;
   
    yPos = _correctionsGroupBox->geometry().bottom() + 10;


    // Create a button group to contain all buttons
    QButtonGroup *sgroup = new QButtonGroup( this, "Photometry method" );
    sgroup->setTitle("Photometry method");
    sgroup->setGeometry(xPos, yPos, 200, 200);
    sgroup->resize( 200, 200 );
    connect( sgroup, SIGNAL(clicked(int)), SLOT(photometryTypeHasChanged(int)) );


   QToolTip::add( sgroup , "The method to be used to compute the photometry of the source - either\n"
                                     "aperture photometry (Aperture button) or least-squares fitting\n"
                                     "to the PSF (either PSF1 Fit1 or PSF Fit2 buttons).\n"
                                     "For either case the background-subtraction can used a fixed\n"
                                     "value for the background/counts per pixel,\n"
                                     "PSF Fit1 will perform a fit, including/excluding a constant term\n"
                                     "for the background. PSF FIT2 does the fit with the source positions\n"
		                     "included in the fit.");





    // Calculate the size for the radio buttons
    maxWidth = 80;
   
    
   
    QFontMetrics fm = sgroup->fontMetrics();

    QString sButtonName[3];
    sButtonName[0] = "Aperture";
    sButtonName[1] = "PSF Fit1";
    sButtonName[2] = "PSF Fit2";
   
 
   


    QString sToolTipName[3];
    sToolTipName[0] = "Aperture Photometry";
    sToolTipName[1] = "Least-squares fit to the PSF";
    sToolTipName[2] = "Least-squares fit to the PSF including fit to the source positions";
  
  


    for (int i = 0; i < 2; i++ ) {
        int w = fm.width( (const char *)sButtonName[i] );
        maxWidth = QMAX(w, maxWidth);   // QMAX is a macro defined in qglobal.h
                                        // and returns the biggest of to values.
        // Due to its macro nature one should use it with care and with
        // constant parameters only.
    }
 

    maxWidth = maxWidth + 20;                      // allow 30 pixels for radiobuttons
    int y0 = 20;

    for (int i=0; i < 3; i++ ) {
        _photometryTypeRadioBtn[i] = new QRadioButton( (const char *)sButtonName[i], sgroup );
        _photometryTypeRadioBtn[i]->setGeometry( 10, y0, maxWidth, 30 );
        y0 += 20;

	QToolTip::add( _photometryTypeRadioBtn[i] , (const char *)sToolTipName[i]);
      
    
       

    }
    


    _photometryTypeRadioBtn[0]->setChecked( true );

    y0 += 20;
    maxWidth += 45;
    sgroup->resize( maxWidth, y0);

    y0 = sgroup->geometry().bottom() - 10;





    _noToFitTxt = new OmSourceLabel( this , "Number to fit", 20, y0, height,
         "Total number of sources to be included in the least-squares fit to the PSF");
  
    QToolTip::add(_noToFitTxt,"Total number of sources to be included in the least-squares fit to the PSF,\n"
                 "including the source itself. The additional sources to be included in the fit are used\n"
		  "in order of their distance from the source.\n"
                  "A maximum of 5 sources can be fit simultaneously");


    /*****************************************************************************
    * A spin-box for setting the number of sources to be included in the PSF fit
    *****************************************************************************/
 
    _noToFitSpinBox = new QSpinBox( this , "SpinBox2" );
    _noToFitSpinBox->setGeometry( _noToFitTxt->geometry().right() + 10, y0, 40, height ); 

    _noToFitSpinBox->setDisabled( true );


 
    _noToFitSpinBox->setMaxValue( min(5, _noSources) );
    _noToFitSpinBox->setMinValue( 1 );
    _numberToFit = 1;


   


    y0 += height + 10;
    /*******************************************************************
    * For determining the weighting method
    *******************************************************************/
    _weightingCheckBox = new QCheckBox( this, "1/variance" );
    _weightingCheckBox->setText("1/counts weights" );
    _weightingCheckBox->setGeometry(20, y0, 120, 30);

    connect(_weightingCheckBox, SIGNAL(toggled( bool )), this, 
                 SLOT ( weightingHasChanged( bool )));

    _weightingCheckBox->setDisabled( true );







    QToolTip::add( _weightingCheckBox , "If checked, then the least-squares fit\n"
                                        "will use weights of 1/pixel counts,\n"
		                        "otherwise unit weighs will be used"); 
 



       
    sgroup->resize( maxWidth, sgroup->geometry().height() + 60);


    _noToFitTxt->setDisabled( true );

   



    xPos = sgroup->geometry().left();
    yPos = sgroup->geometry().bottom() + 10;


    
    _instructionsLabel = new OmSourceLabel(this, " ",  xPos, yPos, 90, 100 ); 
  

    _instructionsLabel->setText("                              ", 20); 









    /*******************************************************************
    * The background method
    *******************************************************************/

    xPos = sgroup->geometry().right() + 10;
    yPos = sgroup->geometry().top();

    y0 = 20;
    OmSourceGroupBox  *backgroundGroupBox = new OmSourceGroupBox( 
               this,  "Background method", xPos, yPos, 230, 140,
	       "The following rates are those obtained from the source photometry\n");




    QToolTip::add( backgroundGroupBox , "The method used to subtract the background counts-\n"
                                        "If the \"Fixed background\" button is checked then\n"
                                        "the background will be subtracted using the value below.\n"
                                        "Otherwise, for aperture photometry the background will\n"
                                        "be subtracted using the pixels within the background\n"
                                        "annulus of either the image or background image (if provided)."
                                        "For PSF fitting, for a non-fixed background the background\n"
                                        "will be obtained from a least-squares fit to the PSF which\n"
                                        "includes a constant term.");

    /*******************************************************************
    * For determining if the background is fixed
    *******************************************************************/
    _fixedBackgroundCheckBox = new QCheckBox( backgroundGroupBox, "Fixed background" );
    _fixedBackgroundCheckBox->setText("Fixed background" );
    _fixedBackgroundCheckBox->setGeometry(10, y0, maxWidth, 30);

    connect(_fixedBackgroundCheckBox, SIGNAL(toggled( bool )), this, 
                 SLOT ( fixedBackgroundHasChanged(bool)));

    _fixedBackgroundCheckBox->setChecked( false );

    QToolTip::add( _fixedBackgroundCheckBox , "If checked, then the background value below\n"
                                      "will be used to subtract the background" );
 


    y0 += 40;



    /***************************************************************************
    * A label to display te current background value
    ****************************************************************************/


    _backgroundTxt = new OmSourceLabel(backgroundGroupBox, "Current value", 10, y0, 30,
         "Current background value");






  
 
    /***************************************************************************
    * An edit box to get the background value
    ****************************************************************************/

    xPos = _backgroundTxt->geometry().right() + 10;
    _backgroundEdt = new OmSourceEdit( backgroundGroupBox, "40000.0", xPos, y0, 80, 30); 



    _backgroundMode = F90MODPROC(omsource_glue, getthebackgroundmode)();
  
    _backgroundUserValue = _backgroundMode;  


    _backgroundEdt->setText(text.sprintf("%8.2f",  _backgroundUserValue  ));

    y0 += 40;

    /*****************************************************************************
    * A button for getting the background from the image
    *****************************************************************************/
    _computeBackgroundBtn = new OmSourceButton(backgroundGroupBox, "&Compute Background", 10, y0, 30, 
               "Click here to get the background from background regions");

    connect( _computeBackgroundBtn, SIGNAL(clicked(  )), this, 
                 SLOT ( showBackgroundDlg()) );







    /*****************************************************************************
    * A button for getting the background from the user
    *****************************************************************************/
    y0 += 40;
    _inputBackgroundBtn = new OmSourceButton(backgroundGroupBox, "&Input Background", 10, y0, 30, 
               "Click here to get the background from background regions");

    connect( _inputBackgroundBtn, SIGNAL(clicked(  )), this, 
                 SLOT ( getBackgroundValue() ) );



    backgroundGroupBox->resize( _backgroundEdt->geometry().right() + 20,
                                _inputBackgroundBtn->geometry().bottom() + 20);
 
 
    /***********************************************************
    * Annuli
    **********************************************************/
    xPos = backgroundGroupBox->geometry().right() + 10;
    yPos = sgroup->geometry().top();

   
    _annuliGroupBox = new OmSourceGroupBox( this,  "Annuli (pixels)", xPos, yPos, 160, 225);
    //    QToolTip::add(_annuliGroupBox, "The following controls allow you to change the aperture\n"
    //                                   "and the radii of the inner and outer background radii\n"
    //                                   "If the source is being treated as extended then you\n"
    //		                   "can also use an elliptical aperture");   

    reporter->setProgress(23);
   
    xPos = 10;
    yPos = 20;
   
    _apertureTxt = new OmSourceLabel(_annuliGroupBox, "Aperture",  xPos, yPos, height);


  

    /*****************************************************************************
     * Need the cal aperture radius (binned) for doing photometry
     ***************************************************************************/
    PhotometryParameters photParameters = F90MODVAR(omsource_glue, myphotometryparameters);
    _calApertureRadius =  photParameters.calApertureRadius; 
    _apertureRadius =  photParameters.apertureRadius;




    if(_nx >= 50)
        _zoomFactor = 3 * photParameters.binX;
  
    else
        _zoomFactor = 0; 




    QTextOStream( &text ) << "The aperture radius (allowed values=" << int(_apertureRadius / 2.0 + 0.1)
                          << " - " 
                          << int(_apertureRadius + 0.1) << endl; 
                         


    QToolTip::add( _apertureTxt, text);
  
 
    xPos += 80; 
    _apertureSpinBox = new QSpinBox(   _annuliGroupBox, "SpinBox2" );
    _apertureSpinBox->setGeometry( xPos, yPos, 50, height ); 

    connect(_apertureSpinBox, SIGNAL(valueChanged(int)), this, SLOT(  apertureRadiusHasChanged(int)));
   

    QToolTip::add( _apertureSpinBox, text);
   
 
    xPos = 10;
    yPos += 30;
    _innerBckTxt = new OmSourceLabel(_annuliGroupBox,  tr( "Inner bck"),  xPos, yPos, height);
 
 
  
    xPos += 80;
    
    _innerBckSpinBox = new QSpinBox(   _annuliGroupBox, "SpinBox2" );
    _innerBckSpinBox->setGeometry( xPos, yPos, 50, height ); 
    reporter->setProgress(24);
   
    connect(_innerBckSpinBox,  SIGNAL(valueChanged(int)), 
           this, SLOT(  innerAnnulusRadiusHasChanged(int)));
    text = "";
    if (_calApertureRadius > 12)   	
        QTextOStream( &text) << "Change the background inner annulus radius\n"
			     << "allowed values=aperture radius + 2 - aperture radius + 32 pixels\n"
                            
                             << "Note that the point-spread function\n"
			     << "extends to " <<   _calApertureRadius << " pixels";
       
    else
        QTextOStream( &text) << "Change the background inner annulus radius\n"
                           << "allowed values=aperture radius + 2 - aperture radius + 32 pixels\n"
                           << "Note that the point-spread function\n"
			   << "extends to " <<   _apertureRadius << " pixels";


       
    QToolTip::add(_innerBckSpinBox, text);
    QToolTip::add(_innerBckTxt, text);

 
    xPos = 10;
    yPos += 30;
 
    _outerBckTxt = new OmSourceLabel(_annuliGroupBox,  tr( "Outer bck"),  xPos, yPos, height);
    reporter->setProgress(25);
   

    xPos += 80;
    _outerBckSpinBox = new QSpinBox( _annuliGroupBox, "SpinBox2" );
    _outerBckSpinBox->setGeometry( xPos, yPos, 50, height ); 

    QToolTip::add( _outerBckSpinBox ,
                  "Background outer annulus radius\n"
                  "(allowed values=inner background radius + 2 - inner background radius + 32 pixels)");
  
    QToolTip::add( _outerBckTxt ,
                  "Background outer annulus radius\n"
                  "(allowed values=inner background radius + 2 - inner background radius + 32 pixels)");
  

    connect( _outerBckSpinBox, SIGNAL(valueChanged(int)), 
           this, SLOT(  outerAnnulusRadiusHasChanged(int)));
	
 
    xPos = 10;
    yPos += 30;

 
    // Get the required source 
    OmSourceParameters& mySources = F90MODVAR( omsource_glue, mysources );   
 
  
    _source = mySources[_sourceNo];
    _oldSource = _source;

 


    _innerBckRadius = int ( _source.innerbackgroundradius + 0.1);
    _outerBckRadius = int( _source.outerbackgroundradius + 0.1);
 
 


    xPos += 80;
 
    _annuliGroupBox->resize(_outerBckSpinBox->geometry().right() + 20, 
                            _outerBckSpinBox->geometry().bottom() + 20); 



    /******************************************************************
    *
    * A spin box to set the photometry aperture colour
    *
    *****************************************************************/
    xPos = _annuliGroupBox->geometry().right() - 70; // was 20
    yPos = _annuliGroupBox->geometry().bottom() + 10;

 
    OmSourceGroupBox *colourBox = new OmSourceGroupBox( this,  "Ds9 display", xPos, yPos, 70);


    OmSourceLabel *label1 = new OmSourceLabel(colourBox, "Aperture", 10, 20);

    _spinBoxColour1 = new OmSpinBox( colourBox, "Change the colour of the aperture region", _colourList.size() - 1, _colourList, _colour1No + 1);
    _spinBoxColour1->setGeometry( label1->geometry().left(), label1->geometry().bottom(), 70, 25);
    connect(_spinBoxColour1, SIGNAL(valueChanged(int)), this, SLOT ( colour1Changed(int)));
    _spinBoxColour1->show();
  



    /******************************************************************
    *
    * A spin box to set photometry background annulus colours
    *
    *****************************************************************/
    
    OmSourceLabel *label2 = new OmSourceLabel( colourBox, "Background", label1->geometry().left(), 
                    _spinBoxColour1->geometry().bottom() + 10);
    _spinBoxColour2 = new OmSpinBox( colourBox, "Change the colour of the background annulus", 
                   _colourList.size() - 1, _colourList, _colour2No + 1);
    _spinBoxColour2->setGeometry( label2->geometry().left(), label2->geometry().bottom(), 70, 25);
    connect(_spinBoxColour2, SIGNAL(valueChanged(int)), this, SLOT ( colour2Changed(int)));

    colourBox->resize( max(colourBox->width(), _spinBoxColour2->geometry().right() + 10),   
                    _spinBoxColour2->geometry().bottom() + 10);


    colourBox->move( _annuliGroupBox->geometry().right() - colourBox->width(), colourBox->geometry().top() );

    _spinBoxColour2->show();
  




    //*************************************************************
    // The compute button
    //*************************************************************
    xPos = backgroundGroupBox->geometry().left();  
    yPos = backgroundGroupBox->geometry().bottom() + 20;
     

    /*****************************************************************************
    * A help button
    *****************************************************************************/
    OmSourceButton *helpButton = new OmSourceButton(this, " &Help ", xPos, yPos, 30, 
               "Click here for help");
  
  
    connect( helpButton, SIGNAL(clicked()), this, SLOT ( showHelpInformation()) );
   
    QToolTip::add( helpButton, "Click here, or click the right mouse-button,\n""for information on the photometry, etc");
  


    xPos = helpButton->geometry().right() + 20;

    _computeBtn = new OmSourceButton(this , "Compute photometry", xPos, yPos, 30,           
        "Click here to recompute the instrumental magnitude\nusing the selected aperture and background annulus");
    
    connect( _computeBtn,SIGNAL(clicked()), this, SLOT(compute()));
 

    _weightingCheckBox->setChecked( true );
  


    reporter->setProgress(29);
   
    /*******************************************************************************
    * create the logger
    ********************************************************************************/
    yPos = colourBox->geometry().bottom() + 5;
    _loggerGroupBox = new OmSourceGroupBox( this, " ",   10, yPos, 510, 250);
   

    _logger = new OmSourceLog(_loggerGroupBox); 
    _logger->setGeometry( 10, 20, 490, 120 );
   
    _averageMagList  = new QComboBox(  _loggerGroupBox);
    _averageMagList->setGeometry(10, 160, 350, 30);
    _averageMagList->setDisabled(TRUE);
    QToolTip::add( _averageMagList , "The average-magnitude table\n"
                                     "This table stores the running average magnitude for the selected source\n"
                                     "Computed from all the values stored in the logger table above\n"
                                     "A new entry is added after the apply button is clicked");
    reporter->setProgress(30);
   
   
    /*********************************************************
    * A button to print its contents
    ********************************************************/
    _printAveragesBtn = new OmSourceButton( _loggerGroupBox, "Print averages",  
                           "Click here to print the contents of\nthe average-magnitude table above");
   
    _printAveragesBtn->setGeometry(10, 200, 100 , 35);
    connect(_printAveragesBtn, SIGNAL(clicked()), this, SLOT (print1()) );
   
                                       
    _printAveragesBtn->setDisabled(TRUE);
    
    /*********************************************************
    * A button to delete average-magnitude table contantes
    ********************************************************/
    //    _deleteAveragesBtn = new OmSourceButton(_loggerGroupBox, "Delete table",
    //                          "Click here to delete the contents of\n"
    //                          "the average-magnitude table above" );
  
    // _deleteAveragesBtn->setGeometry(130, 200, 100, 35);
    // connect(_deleteAveragesBtn, SIGNAL(clicked()), this, SLOT (deleteAverageMagList()) );
   
		
    reporter->setProgress(31);
                
    //    _deleteAveragesBtn->setDisabled(TRUE);
   
 
    /*********************************************************
    * A button to print the logger
    ********************************************************/
    _printLogBtn = new OmSourceButton(_loggerGroupBox,  "Print table", 
		     "Click here to print the entries in the logger table above");
    _printLogBtn->setGeometry(_loggerGroupBox->width() - 110, 160, 100 , 35); 

    connect(_printLogBtn, SIGNAL(clicked()), this, SLOT (print2()) );
   
    _printLogBtn->setDisabled(TRUE);
    

    
    /*********************************************************
    * A button to delete average-magnitude table contantes
    ********************************************************/
    _deleteAveragesBtn = new OmSourceButton(_loggerGroupBox, "Delete table",
                              "Click here to delete the contents of\n"
                              "the average-magnitude table above" );
  
    _deleteAveragesBtn->setGeometry(_printLogBtn->geometry().left(), _printLogBtn->geometry().bottom() + 5, 100, 35);
    connect(_deleteAveragesBtn, SIGNAL(clicked()), this, SLOT (deleteAverageMagList()) );
   

    _deleteAveragesBtn->setDisabled(TRUE);




    reporter->setProgress(32);
   
    LoadSourceDetails();

 
    _source.rawbackgroundcountrate = -1.0;
    
   
  

  

    reporter->setProgress(33);
   
    // NB In the source-list file the aperture-radius is stored as unbinned pixels
    // Want to display it in the table as actual extraction radius
  
   

 

    // Set the first entry in the logger table with the pre-computed parameters
    _logger->add( _oldSource );


    _magnitude = new float[1000];
    _magnitudeError = new float[1000];
    _count = 0;

    computeAverage();
    _computeBtn->setDisabled(true);




    /**********************************************************
    * Exit and cancel buttons
    **********************************************************/    
    yPos = this->height() - 50;
    xPos = this->width() / 3;




    _finishButton = new OmSourceButton( this, "Okay",
         "Click here to return to the main window");
      
    _finishButton->setGeometry(xPos, yPos, 70, 35);

    _finishButton->setDisabled(true);
    connect(_finishButton, SIGNAL(clicked()), this, SLOT(accept()));
    reporter->setProgress(34);
   

 
    xPos += 90;
  
    _cancelButton = new OmSourceButton( this, "Cancel", 
        "Click here to keep the original source parameters and return to the main window");
    _cancelButton->setGeometry(xPos, yPos, 70, 35);
    connect(_cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
   
   
  
    reporter->setProgress(35);
   


    reporter->setProgress(36);

    
    reporter->setProgress(37);
 
 
    /***************************************************************   
    *
    ***************************************************************/
    // add an information  menu:
    QPopupMenu* infoMenu = new QPopupMenu;
    
    infoMenu->insertItem("&Help", parent, SLOT(showHelpInformation()) );
 
    infoMenu->insertItem( "&Obs details",  parent, 
          SLOT(showObservationDetails()), Qt::CTRL+Qt::Key_O );
 
    infoMenu->insertItem( "&Image details",  parent, 
          SLOT(showImageDetails()), Qt::CTRL+Qt::Key_I );

    reporter->setProgress(38);

    QMenuBar* menuBar = new QMenuBar(this);
    menuBar->insertItem("&Information", infoMenu);
    reporter->setProgress(39);

    // Get the background mode from the image

     reporter->setProgress(40);

 



     //   setGeometry(0, 0, _loggerGroupBox->geometry().right() + 10, _cancelButton->geometry().bottom() + 10);      

    move(0, 0);
    resize( _loggerGroupBox->geometry().right() + 10, _cancelButton->geometry().bottom() + 10);    


    reporter->setProgress(41);



    initApertureSpinBoxes();
 
    _fixedBackground = false;




    connect( _noToFitSpinBox, SIGNAL( valueChanged( int )), this, 
                 SLOT ( numberToFitHasChanged( int )) );

    numberToFitHasChanged( 1 );



    delete reporter;

    _timer = new QTimer(this);
    connect(_timer, SIGNAL(timeout()), SLOT(updateDisplay()) );
 
    photometryTypeHasChanged( 0 ); // Initialise with aperture photometry

    
}
/**************************************************************
*  
*  Destroys the object and frees any allocated resources
**************************************************************/
FastPhotometryDlg::~FastPhotometryDlg()
{
    delete [] _magnitude;
    delete [] _magnitudeError;


    // Delete the background regions
    F90MODPROC(omsource_glue, deletebackgroundlist)();
      
    // no need to delete child widgets, Qt does it all for us
  
}
/*********************************************
* cancel -
*  
*********************************************/

void FastPhotometryDlg::cancel()
{

    F90MODPROC(omsource_glue, replace)(_sourceNo, _oldSource);
      
    reject();

}
/***********************************************
* 
*************************************************/
void FastPhotometryDlg::LoadSourceDetails()
{
 
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
    _source = mySources[_sourceNo];
 



 
    QString text;
  
    float totalRate =  _source.rawcountrate +  _source.rawbackgroundcountrate;
    float error;
   
    if (_source.rawbackgroundcountrateerror > 0.0)
        error =  sqrt(_source.rawcountrateerror * _source.rawcountrateerror +
		       _source.rawbackgroundcountrateerror * _source.rawbackgroundcountrateerror);
    else
    {
      error = 0.0;
      _source.rawbackgroundcountrateerror = 0.0;

    }
    _rawTotalRateEdt->setText(text.sprintf("%8.2f",  totalRate));
  
    _rawTotalRateErrEdt->setText(text.sprintf("%8.2f",  error));;
  
    _rawBackgroundRateEdt->setText(text.sprintf("%8.2f",  _source.rawbackgroundcountrate));

    _rawBackgroundRateErrEdt->setText(text.sprintf("%8.2f",  _source.rawbackgroundcountrateerror));

 
    _rawBackgroundSubtractedRateEdt->setText(text.sprintf("%8.2f",  _source.rawcountrate));

    _rawBackgroundSubtractedRateEdt->setText(text.sprintf("%8.2f", _source.rawcountrate  ));
  

     
   
    _rawBackgroundSubtractedRateErrEdt->setText(text.sprintf("%8.2f",_source.rawcountrateerror));
  
    
    // Corrected count rates

    totalRate =  _source.corrcountrate +  _source.corrbackgroundcountrate;
  
    //    cout << "Line 1098 raw count rate=" <<  _source.rawcountrate << " error=" <<  _source.rawcountrateerror << endl;
    //  cout << "Line 1098 corr count rate=" <<  _source.corrcountrate << " error=" <<  _source.corrcountrateerror << endl;
    //   cout << "Line 1098 background raw count rate=" <<  _source.rawbackgroundcountrate << " error=" <<  _source.rawbackgroundcountrateerror << endl;
    //  cout << "Line 1098 background corr count rate=" <<  _source.corrbackgroundcountrate << " error=" <<  _source.corrbackgroundcountrateerror << endl;


    if (_source.corrbackgroundcountrateerror > 0.0)
        error =  sqrt(_source.corrcountrateerror * _source.corrcountrateerror +
		       _source.corrbackgroundcountrateerror * _source.corrbackgroundcountrateerror);
    else
    {
        error = 0.0;
        _source.corrbackgroundcountrateerror = 0.0;
    }

    _corrTotalRateEdt->setText(text.sprintf("%8.2f",  totalRate));
  

    _corrTotalRateErrEdt->setText(text.sprintf("%8.2f",  error));;
  
    _corrBackgroundRateEdt->setText(text.sprintf("%8.2f",  _source.corrbackgroundcountrate));
    _corrBackgroundRateErrEdt->setText(text.sprintf("%8.2f",  _source.corrbackgroundcountrateerror));


 
    _corrBackgroundSubtractedRateEdt->setText(text.sprintf("%8.2f", _source.corrcountrate));
    _corrBackgroundSubtractedRateErrEdt->setText(text.sprintf("%8.2f", _source.corrcountrateerror));
  
    _magnitudeEdt->setText(text.sprintf("%8.2f",  _source.magnitude));
    _magnitudeErrEdt->setText(text.sprintf("%8.2f",  _source.magnitudeerror));
    _signalToNoiseEdt->setText(text.sprintf("%8.2f",  _source.significance));      





    /******************************************************
    * Compute the AB magnitude anf flux
    ******************************************************/
    float ABMag = -1.0, ABMagErr = -1.0, ABFlux = -1.0, ABFluxErr = -1.0;

    computeABMagsAndFluxes(_source.corrcountrate, _source.corrcountrateerror, 
                           ABMag, ABMagErr, ABFlux, ABFluxErr);

    _ABMagnitudeEdt->setText(text.sprintf("%8.2f",  ABMag));
    _ABMagnitudeErrEdt->setText(text.sprintf("%8.2f",  ABMagErr));

    _ABFluxEdt->setText(text.sprintf("%10.5e",  ABFlux));
    _ABFluxErrEdt->setText(text.sprintf("%10.5e",  ABFluxErr));



    /******************************************************************************
    *
    *******************************************************************************/
    float deadFractionCorrection = 1.0 / (1.0 - _imageDetails->deadFraction);


    // Coincedence-loss corrections
    _coincCorrectionEdt1->setText(text.sprintf("%5.3f",  _source.semiaerr * deadFractionCorrection));

    _coincCorrectionEdt2->setText(text.sprintf("%5.3f",  _source.semiberr * deadFractionCorrection));


    //    cout << "Line 1410 psf1 correction=" <<  _source.psf1correction << endl;
    _psf1CorrectionEdt->setText(text.sprintf("%5.3f",  (1.0 + _source.psf1correction / 100.0)));
 

    //    if (_obsDetails->filter == "UVW1" || _obsDetails->filter == "UVW2" ||  _obsDetails->filter == "UVM2")
    {
     _psf2CorrectionEdt->setText(text.sprintf("%5.3f",  (1.0 + _source.psf2correction / 100.0)));

    
    }

    //    cout << "Omsourcephotometrydlg.cc line 1416 deadfraction=" << _imageDetails->deadFraction << endl;
  
    _tdsCorrectionEdt->setText(text.sprintf("%5.3f",  _source.tdscorrection));

}
/******************************************************
* initApertureSpinBoxes
* Purpose- Initialises the spin boxes with the default
* values
******************************************************/

void FastPhotometryDlg::initApertureSpinBoxes()
{




    /************************************************************************
    *The aperture radius
    * Need to change the tool-tips
    *************************************************************************/
    QToolTip::remove( _apertureSpinBox );
    QToolTip::remove( _apertureTxt );
  

    QString text; 
   


    _apertureTxt->setText( "Aperture" );

    _apertureSpinBox->setMaxValue(int(_apertureRadius + 0.1));
    _apertureSpinBox->setMinValue(int(_apertureRadius / 2.0 + 0.1));
   



    _apertureSpinBox->setValue((int)(_apertureRadius));
  




    QTextOStream( &text ) << "The aperture radius (allowed values=" << _apertureSpinBox->minValue()
                          << " - " 
                          << _apertureSpinBox->maxValue() << " pixels)"; 
                      
    QToolTip::add( _apertureTxt, text);
    QToolTip::add( _apertureSpinBox, text);

  
    // The inner background annulus  
    _innerBckSpinBox->setMinValue( _apertureSpinBox->value()    );
    _innerBckSpinBox->setValue(   (int)_innerBckRadius   );
    _innerBckSpinBox->setMaxValue( int(_innerBckRadius) + 20 );




    // The outer background annulus  
    _outerBckSpinBox->setMinValue(_innerBckSpinBox->value() + 2);
       
    _outerBckSpinBox->setValue( (int)_outerBckRadius );
    _outerBckSpinBox->setMaxValue( (int)_outerBckRadius + 20);
 
 
   
  

    setToolTips();   
   
}
/******************************************************
* sourceHasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void FastPhotometryDlg::apertureRadiusHasChanged(int value)
{
     
    _computeBtn->setEnabled(true);
    
    _source.photometryradius = value;



    _innerBckSpinBox->setMinValue( value );
    
    if (value > _innerBckSpinBox->minValue() )
    {
   
        _innerBckSpinBox->setMaxValue( value + 30 );
        _innerBckSpinBox->setValue( value);
    }
    setToolTips();

    //    cout << "Line 1487 aperture radius has changed line 1487\n";

    startTimer();
}
/******************************************************
* 
******************************************************/
void FastPhotometryDlg::setToolTips()
{

    QString text1, text2;
       
    QTextOStream( &text1 ) << "The inner background annulus radius (allowed values=" << _innerBckSpinBox->minValue()
                          << " - " 
                          << _innerBckSpinBox->maxValue() << " pixels)"; 
                      
 
    QTextOStream( &text2 ) << "The outer background annulus radius (allowed values=" << _outerBckSpinBox->minValue()
                          << " - " 
                          << _outerBckSpinBox->maxValue() << " pixels)"; 
                      
   
    QToolTip::remove( _innerBckSpinBox );
    QToolTip::remove( _innerBckTxt );
    QToolTip::add( _innerBckTxt, text1);
    QToolTip::add( _innerBckSpinBox, text1);

    QToolTip::remove( _outerBckSpinBox );
    QToolTip::remove( _outerBckTxt );
    QToolTip::add( _outerBckTxt, text2);
    QToolTip::add( _outerBckSpinBox, text2);



}

/******************************************************
* 
******************************************************/
void FastPhotometryDlg::startTimer()
{

  //  cout << "start timer line 1546\n";
    if (_timer == 0) return;

    if (_timer->isActive())
    {
        _timer->stop();
    }
    _timer->start(1000, FALSE ); // One second timer
    
    //    cout << "line 1554 timer started\n";
}
/******************************************************
* innerAnnulusRadiusGasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void FastPhotometryDlg::innerAnnulusRadiusHasChanged(int value)
{
  
    _computeBtn->setEnabled(true);
 
    _outerBckSpinBox->setMinValue( value + 2);
    


    _outerBckSpinBox->setValue( max( _outerBckSpinBox->value(), value + 5) );

    _outerBckSpinBox->setMaxValue( max(_outerBckSpinBox->value() + 5, _outerBckSpinBox->maxValue()) );


    _source.innerbackgroundradius = value;




    setToolTips();
    startTimer();
    
      
       
}
/******************************************************
* outerAnnulusRadiusGasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void FastPhotometryDlg::outerAnnulusRadiusHasChanged(int value)
{  
    _computeBtn->setEnabled(true);
    _source.outerbackgroundradius = value;

  
    setToolTips();

    startTimer();
    
       
     
}

/*********************************************************
 *
 ********************************************************/
void FastPhotometryDlg::updateDisplay()
{
       
    _timer->stop();

  
  
    _source.number = _sourceNo;
    _source.photometryradius = _apertureSpinBox->value();


    _source.innerbackgroundradius = _innerBckSpinBox->value();
    _source.outerbackgroundradius = _outerBckSpinBox->value();
 
 
    //   _display->zoomTo( _source, _zoomFactor, _colour1.c_str(), _colour2.c_str() );


   

    if ( _photometryType == 0)
    {

        _display->zoomTo( _source, _zoomFactor, _colour1.c_str(), _colour2.c_str() );

    }
    else
    {
        _display->zoom1To( _source, _zoomFactor, _colour1.c_str());
   
    }

    _display->displayCurrentSourceList1( _oldRegionsColour.c_str(), false );

 

    _display->displayBackgroundRegions( BackgroundDlg::_backgroundList, OmSourceBrowser::_backgroundRegionsColour.c_str() );

    

    OmSourceRegions *points1 = OmImageRegions::GetNewRegions( _display, _nx, _ny );
     


    if (points1) 
    {
        delete(points1);
        points1 = 0;	  
    }




}
/******************************************************
* recompute
* This is activated from OmSourceEditor when the apply
* button is clicked.
*******************************************************/
void FastPhotometryDlg::compute()
{

    int method = 0;

    int noTerms = _numberToFit;

    int fitNo[10];
    fitNo[0] = _sourceNo; 


    //    cout << "Compute line 1827\n";
    // cout << "omfastphotetryDlg.cc line 1822 noTerms=" << noTerms << " sourceNo=" << _sourceNo << endl;


    //  for(int i = 0; i < noTerms; i++)
    // { 
    //     cout << i << ") fit number=" << _fitNo[i];	     
    //  }



   _instructionsLabel->setText("Computing photometry\n" 
                               "Please wait", 20); 



    _computeBtn->setDisabled(true);


    qApp->processEvents();
 
    //    QTimer *timer = new QTimer(this);
    //    connect( timer, SIGNAL( timeout() ), this, SLOT( updateCputime()) );
    //    timer->start(1000);  

    //   timer->start(0, false);  


    //    _time.start();





    switch( _photometryType )
    {
        case(0): // Aperture Photometry

            _source.photometryradius = _apertureSpinBox->value();

         	   
	    if(_fixedBackground)     
	    {
	        method = 1;
                _source.rawbackgroundcountrate = _backgroundUserValue;

                _source.iMin = -3;
            }
            else
	    {
	        method = 0;
                _source.iMin = -2;
       
                _source.rawbackgroundcountrate = -1.0;
            }

	    break;

        case(1): // PSF fit with no fit to the positions
	      
	  //	 cout << "OmFastPhotometry line 1873 Psf fitting - no fit to positions to " << noTerms << " sources\n";
	  // cout << "Source number=" << _sourceNo << endl;
            for(int i = 0; i < noTerms; i++)
	    { 
	      fitNo[i] = _fitNo[i];
	      //    cout << "Line 1878 Psf fitting to source " << fitNo[i] << endl; 


	    }

	    if(_fixedBackground) 
	    {
	        method = 3;
	    
                _source.rawbackgroundcountrate = -1.0;

                _source.iMin = -3;
       
            }
            else
	    {
	        method = 2;
                _source.iMin = -4;
       
            }
	    break;

        case(2): // PSF fit with a fit to the positions
	      
	  //           cout << "Psf fitting with fit to position sto " << noTerms << " sources\n";
            for(int i = 0; i < noTerms; i++)
	    { 
	      fitNo[i] = _fitNo[i];
	      //		  cout << "Psf fitting to source " << fitNo[i] << endl; 


	    }

	    if(_fixedBackground) 
	    {
	        method = 5;
	    	_source.iMin = -3;
   

                _source.rawbackgroundcountrate = -1.0;


            }
            else
	    {
	        method = 4;
                _source.iMin = -4;
   
            }
	    break;


        default:
	    break;
    }


  


   
    double radius = double(_apertureSpinBox->value());
    double innerA = double(_innerBckSpinBox->value());
    double outerA = double(_outerBckSpinBox->value());
    double axialRatio = 1.0;
    double theta = 0.0;
    double background = double(_backgroundUserValue);
    //    cout << "omfastphotometrydlg.cc line 1786 radius=" << radius << endl;


    //    cout << "omfastphotometry line 1948 - photometrytype=" << _photometryType << " method=" << method << " noterms=" << noTerms << endl;





    F90MODPROC(omsource_glue, replace)( _sourceNo, _source);
 

    //    cout << "Line 1957\n";
    F90MODPROC( omsource_glue, recompute_photometry )( 
                  &noTerms, 
	          fitNo, 
	          &method,
	          &radius, 
	          &innerA, 
	          &outerA, 
	          &axialRatio, 
                  &theta,
                  &background);

    //    cout << "Line 1969\n";

  
    _finishButton->setEnabled(true);
 

    // Retrieve the new source parameters

   
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
    



    _source = mySources[_sourceNo];





    _logger->add(_source );
 

 


    LoadSourceDetails();

  


    _averageMagList->setEnabled(true);
    computeAverage();

   


    _instructionsLabel->setText("        ", 20); 


    //    timer->stop();
    //    delete timer; 
 
 

    
}
/******************************************************
* print
******************************************************/
void FastPhotometryDlg::print()
{
#ifndef QT_NO_PRINTER

    QPrinter printer; 
    if ( printer.setup(this) ) {                // opens printer dialog
        printer.setFullPage(TRUE);              // we'll set our own margins
	QPainter p;
	p.begin( &printer );                    // paint on printer
	p.setFont( p.font() );
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( &printer ); // need width/height
                                                 // of printer surface
    
        /*********************************************
	* Print what ever
	********************************************/
        (this->*Print)(&printer, &p,  &fm,  &metrics);

	p.end();                                // send job to printer
    }
#endif
}

/*********************************************************
* print1
**********************************************************/
void FastPhotometryDlg::print1()
{



    _timer->stop();


    Print = &FastPhotometryDlg::printAverages;
    print();

   startTimer();


}
/*********************************************************
* print2
**********************************************************/
void FastPhotometryDlg::print2()
{   

   _timer->stop();



  
    Print = &FastPhotometryDlg::printLog;
    print();


   startTimer();



}

/*********************************************************
* printAverages
* Prints the contents of the combo box that stores
* the average magnitudes
*********************************************************/
void FastPhotometryDlg::printAverages(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line


    printHeading(p,  fm, metrics, MARGIN, yPos);
 

    for( int i = 0 ; i < _averageMagList->count(); i++ ) 
    {
        if ( printer->aborted() )
            break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
                if ( !printer->newPage() )          // start new page
                    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }
        p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, _averageMagList->text(i));
        yPos += fm->lineSpacing();
    }

}

/*****************************************************************************
*
* printHeading
****************************************************************************/
void FastPhotometryDlg::printHeading(QPainter* p,  
                                  QFontMetrics* fm,  
                                  QPaintDeviceMetrics* metrics,
                                  const int MARGIN, int &yPos)
{
    QDateTime date = QDateTime::currentDateTime();  // The current date and time
    
    QString text = QString("Listing produced by ") + omsourceVersionNo() + QString("  ") + date.toString();  
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 2 * fm->lineSpacing();
   
    text = QString("TELESCOPE=") + _obsDetails->telescope + QString(" INSTRUMENT=") + _obsDetails->instrument;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("OBS_ID=") + _obsDetails->obsId + QString(" EXP_ID=") + _obsDetails->expId;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("DATE-OBS=") + _obsDetails->dateObs + QString(" DATE-END=") + _obsDetails->dateEnd;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("OBJECT=") + _obsDetails->object + QString(" OBSERVER=") + _obsDetails->observer;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text.sprintf("FILTER=%s EXPOSURE=%d (SECONDS)", (const char *)_obsDetails->filter, 
           (int)_obsDetails->exposure);
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 5 * fm->lineSpacing();

}
/*********************************************************
* printLog
* Prints the contents of the table that stores the
* user photometry calculations
*********************************************************/
void FastPhotometryDlg::printLog(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line

    printHeading(p, fm, metrics, MARGIN, yPos);


    QString text;
    text.sprintf("Source no %d - photometry calculations", _sourceNo);
    p->drawText (MARGIN, yPos, text); 
    yPos += 2 * fm->lineSpacing();
    QString heading[] = {"Photometry aperture radius (unbinned pixels)",
                         "Photometry inner background annulus radius (unbinned pixels)",
                         "Photometry outer background annulus radius (unbinned pixels)",
                         "Instrumental magnitude (corrected for coincidence losses & psf corrections)",
                         "Instrumental magnitude error",
                         "Signal-to-noise ratio)",
                         "Coincidence-loss correction (%)"}; 
 
    for (int i = 0; i <  _logger->numCols(); i++)
    {
        QString text;

        text.sprintf("Column %d=%s", i + 1, (const char *)heading[i]); 
        p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);

        yPos += fm->lineSpacing();
        
    }
     
    yPos += fm->lineSpacing();
   
    for( int i = 1 ; i < _logger->numRows(); i++ ) 
    {
        if ( printer->aborted() )
            break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
                if ( !printer->newPage() )          // start new page
                    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }

        QString text;
        for (int j = 0; j < _logger->numCols(); j++)
	{
	  text += _logger->text(i, j) + QString("        ");
           

        }
   
       p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);


        yPos += fm->lineSpacing();
    }

}
/*********************************************
 *
 *******************************************/
void FastPhotometryDlg::deleteAverageMagList()  
{ 
    _averageMagList->clear();
    
    _logger->setNumRows(0);

    _printAveragesBtn->setDisabled(true);
    _deleteAveragesBtn->setDisabled(true);
    _count = 0;
    repaint();
}; 


/******************************************************
* edit
*******************************************************/

void FastPhotometryDlg::computeAverage()
{
    
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
     
    // Recompute the average magnitude 
      
    _magnitude[_count] = mySources[_sourceNo].magnitude;
    _magnitudeError[_count] = mySources[_sourceNo].magnitudeerror;
    
    if (_count > 0) 
    {

        float mean, standardDeviation;
       
        ComputeAverageMagnitude(_magnitude, _magnitudeError, _count + 1, mean, standardDeviation);
        QString text;
        text.sprintf("Average magnitude for source %d=%5.2f+/-%5.2f (n=%d)", 
               _sourceNo, mean, standardDeviation, _count + 1); 
        
        _averageMagList->setEnabled(true);

        _printLogBtn->setEnabled(true);
        _printAveragesBtn->setEnabled(true);
        _deleteAveragesBtn->setEnabled(true);


        _averageMagList->insertItem(text);
        _averageMagList->setEditText(text);

        _averageMagList->setCurrentItem(_averageMagList->count() - 1); 
    }
    _count++;

     
}



/****************************************************************
*
***************************************************************/

void FastPhotometryDlg::showPhotometryInformation()
{

   
  static const char* sayings[] = {
    //    static const QString sayings[] = {
     "<center><i>Overview of photometry</i></center>",

    "<b>Step 1 - Aperture Counts:</b><br>"
    "<hr><br><br>"
    "<big>Sum up the pixel counts within the aperture </big><br><br>"
    "<center><i>-- H.L. Mencken</i></center>",

    "<b>Step 2- Background Estimation:</b><br>"
    "<hr><br><br>"
    "<big>Obtain an estimate of the background<br>"
    "from the pixels within the background aperture<br>"
    
    "These are great mysteries.</big><br><br>"
    "<center><i>-- Geoffrey James, \"The Tao of Programming\"</i></center>",

    "<b>Obtain the Source Counts:</b><br>"
    "<hr><br><br>"
    "<big>Subtract the background from the total counts</big><br><br>"
    "<center><i>-- Martin Mull</i></center>",

    "<b>Saying 4:</b><br>"
    "<hr><br><br>"
    "<big><b>The Least Successful Executions</b><br>"
    "<twocolumn><p>      History has furnished us with two executioners worthy of attention. "
    "The first performed in Sydney in Australia.  In 1803 three attempts were "
    "made to hang a Mr. Joseph Samuels.  On the first two of these the rope "
    "snapped, while on the third Mr. Samuels just hung there peacefully until he "
    "and everyone else got bored.  Since he had proved unsusceptible to capital "
    "punishment, he was reprieved.</p>"
    "<p>        The most important British executioner was Mr. James Berry who "
    "tried three times in 1885 to hang Mr. John Lee at Exeter Jail, but on each "
    "occasion failed to get the trap door open.<!p>"
    "<p>        In recognition of this achievement, the Home Secretary commuted "
    "Lee's sentence to \"life\" imprisonment.  He was released in 1917, emigrated "
    "to America and lived until 1933.</p></twocolumn></big><br><br>"
    "<center><i>-- Stephen Pile, \"The Book of Heroic Failures\"</i></center>",

    "<b>Saying 5:</b><br>"
    "<hr><br><br>"
    "<big>If you can, help others.  If you can't, at least don't hurt others.</big><br><br>"
    "<center><i>-- the Dalai Lama</i></center>",

    "<b>Saying 6:</b><br>"
    "<hr><br><br>"
    "<big>Television has brought back murder into the home -- where it belongs.</big><br><br>"
    "<center><i>-- Alfred Hitchcock</i></center>",

    "<b>Saying 7:</b><br>"
    "<hr><br><br>"
    "<big>I don't know who my grandfather was; I am much more concerned to know "
    "what his grandson will be.</big><br><br>"
    "<center><i>-- Abraham Lincoln</i></center>",

    0
};


    HelpDlg *helpDlg = new HelpDlg(this, sayings);


    

    helpDlg->exec();
    
     
   
 
}






/***************************************************
* Displays the background dialogue that
* allows the user to select circular regions
* for background determination
***************************************************/
void FastPhotometryDlg::showBackgroundDlg()
{
   


 

    BackgroundDlg *backgroundDlg = new BackgroundDlg(this, _display, &_source, _zoomFactor);
    



 
    backgroundDlg->exec();
    

    _backgroundUserValue = backgroundDlg->getBackgroundMode();  


    QString text;
    text.sprintf("%8.2f",  _backgroundUserValue);  
    _backgroundEdt->setText( text );






    _backgroundRegionsValue = _backgroundUserValue;   
      
  



 
}
/******************************************************
 *
 *****************************************************/
void FastPhotometryDlg::stopTimer()
{
  if (_timer == 0) return;
   _timer->stop();
   // cout << "Line 2062 stopTimer aperture radius=" << _source.photometryradius << endl;
  
}


/************************************************************
*
***********************************************************/
float FastPhotometryDlg::signalToNoise(const float & source, const float &background, const float &exposure)
{

    float s = source * exposure;
    float b = background * exposure;

    return s / sqrt(s + 2.0 * b);


}
/****************************************************************
*
****************************************************************/
void FastPhotometryDlg::colour1Changed( int no )
{
    _colour1No = no;
    _colour1 = _colourList[no - 1];

    startTimer();

}

/****************************************************************
*
****************************************************************/
void FastPhotometryDlg::colour2Changed( int no )
{
    _colour2No = no;
 
    _colour2 = _colourList[no - 1];

    startTimer();

}






//#include <cmath>
//#include <cstdlib>
//#include <string>

//#include "common.h"
#include "CalServer.h"
//#include "Celestial.h"
//#include "Ccf.h"
//#include "Image.h"



#include "CalOalUtility.h"


//#include "errstr.h"
//#include "f90cxx.h"
//#include "Dal.h"
//#include "Handle.h"
//#include "STime.h"

//#include "BadPixelDataServer.h"
//#include "CrossPsf.h"
//#include "CcdQuantumDataServer.h"
//#include "CtiCorrector.h"
//#include "EffectiveAreaDataServer.h"
//#include "EnergyCorrector.h"
//#include "GeometryDataServer.h"
//#include "MiscDataServer.h"
//#include "OmGeoLinearizer.h"
//#include "OmGrismDataServer.h"
//#include "PsfDataServer.h"
//#include "Redistribution.h"
//#include "TimeCorrector.h"

//#include <vector>

using namespace CalOalUtility;

//using namespace std;




#include "Dal.h"
#include "param.h"
#include <iostream>

/******************************************************
*
******************************************************/
void FastPhotometryDlg::photometricConstants()
{

    //  create an OM engine




  //    calServer.setInstrument(Xmm::OM);

  //    if(filter == "UVW2" || filter == "uvw2")
  //        calServer.state()->filter()->set(Xmm::UVW2);
  //    else if(filter == "UVM2" || filter == "uvm2")
  //        calServer.state()->filter()->set(Xmm::UVM2);
  //    else if(filter == "UVW1" || filter == "uvw1")
  //        calServer.state()->filter()->set(Xmm::UVW1);
  //    else if(filter == "U" || filter == "u")
  //        calServer.state()->filter()->set(Xmm::U);
  //    else if(filter == "B" || filter == "b")
  //        calServer.state()->filter()->set(Xmm::B);
  //    else if(filter == "V" || filter == "v")
  //        calServer.state()->filter()->set(Xmm::V);
  //    else if(filter == "GRISM1" || filter == "GRISM1")
  //        calServer.state()->filter()->set(Xmm::GRISM1);



    OmColorTransformator *ct = 0;
    ct = calServer.getAtom( ct );


    OmColorTransformator::OmFluxConvF f = ct->fluxConversionFactors();


    //  cout << "AB Magnitude zero-point=" << f.zpab << endl;
    //  cout << "AB Flux conversion factor=" << showpoint << setw(15) << f.cfab << endl;


    //  cout << "VEGA Flux conversion factor=" << showpoint << setw(15) << f.cfvega << endl;
    //  cout << "WHITE DWARF  Flux conversion factor=" << showpoint << setw(15) << f.cfwd << endl;


}

/******************************************************
*
******************************************************/
void FastPhotometryDlg::computeABMagsAndFluxes(float rate, float rateErr, 
                     float &mag, float &magErr, float &flux, float &fluxErr)
{

    if(rate <= 0.0) 
    {

      return;
    }   

    OmColorTransformator *ct = 0;
    ct = calServer.getAtom( ct );


    OmColorTransformator::OmFluxConvF f = ct->fluxConversionFactors();

    
    /*************************************************************
    * Compute the AB magnitude from the count-rate
    *
    *************************************************************/

    mag =  f.zpab - 2.5 * log10( rate );            
       
    magErr = max(0.01, 1.085735 * rateErr / rate);

    flux =  rate * f.cfab;            
    fluxErr = rateErr *  f.cfab;        
                
}


/******************************************************
*
******************************************************/
    
void FastPhotometryDlg::showHelpInformation()
{
 

    static const char* information[] = {
  
    "<b>APERTURE PHOTOMETRY</b><br>"
    "<hr><br><br>"              
    "The source counts are obtained using the given aperture size.<br><br>" 
    "<b>BACKROUND DETERMINATION</b>- There are three different ways of subtracting the background-. <br><br>"
    "<b>1)</b> By computing a  value of the mean background-counts per pixel by using the pixels within an<br>"
    "annulus region centred on the source. Please note that if a <b>background</b> image is specified, the<br>"
    "background counts will be obtained from this image, rather than the actual image. The <b>background</b>"
    "may be an image from the <b>IMAGING</b> chain containing the source.<br>"
    "<b>2)</b> By using <b>user-selected</b> regions to obtain a value for the mean (<b>Compute background</b> button)<br>"
    " background-counts per pixel.<br><br>"
    "<b>3)</b> By using  a  <b>user-supplied</b> value for the mean background-counts per pixel (<b>Input background</b> button).<br><br>"
     
 
    "<b>PSF PHOTOMETRY</b><br>"
    "<hr><br><br>"
    "PSF photometry utilises the calibration point-spread function to fit a model consisting of one or<br>"
    "more PSF sources to the actual sources to be included in the fit.<br>"
    "The <b>PSF fit1</b> determines the \"heights\" of the sources and the background level (unless this<br>"
    " value is \"fixed\"<br>"
    "The total number of sources to be included in the fit is set by the value in the spin-box \"Fit no\" spin box.<br>"
    "The list of sources is stored in order of increasing distance from the \"current\" source. Only sources<br>"
    "within a radius of 24 unbinned pixels from the primary source will be included in the fitting process.<br>" 
    "The <b>PSF fit2</b> method also includes terms for the source-coordinates<br>"
    "<b>PLEASE NOTE THAT THE FIT USING THE PSF Fit2  METHOD MAY TAKE A CONSIDERABLE WHILE</b>.<br><br>"   
              
    "<b>BACKROUND DETERMINATION</b>- There are two different ways of subtracting the background-. <br><br>"
    "<b>1)</b> By using <b>user-selected</b> regions to obtain a value for the mean (<b>Compute background</b> button<br>"
    " background-counts per pixel.<br><br>"
    "<b>2)</b> By using  a  <b>user-supplied</b> value for the mean background-counts per pixel (<b>Input background</b> button).<br><br>",
    
    0};     
                    
 
    HelpDlg *helpDlg = new HelpDlg( this, information);


    helpDlg->exec();
         
    delete helpDlg;  
 
}
/*************************************************************
*
************************************************************/
void FastPhotometryDlg::mousePressEvent( QMouseEvent* event)
{
  
    if ( event->button() == RightButton )
    {   

        showHelpInformation();
    }



}
/****************************************************************
* 
*
****************************************************************/

void FastPhotometryDlg::photometryTypeHasChanged( int type)
{

    _photometryType = type;
   
  
    //    cout << "photometryhaschanged type=" << type << endl;

    if(type == 0) // Aperture photometry
    {

        _apertureSpinBox->setEnabled( true );
        _apertureTxt->setEnabled ( true );
  
          _noToFitTxt->setDisabled( true );
          _noToFitSpinBox->setDisabled( true );
  
          _spinBoxColour1->setEnabled( true );
          _spinBoxColour2->setEnabled( true );




          _weightingCheckBox->setDisabled( true ); 

         _numberToFit = 1; // The total number of sources in the least-squares PSF fit


         _fitNo.clear();
   

   
   
      
    }
    else // Least-squares fit to point-spread function - no need for apertures
    {

        _apertureSpinBox->setEnabled( false );

        _apertureTxt->setEnabled ( false );

        _weightingCheckBox->setEnabled( true ); 

        _spinBoxColour1->setDisabled( true );
        _spinBoxColour2->setDisabled( true );

	//     _fitNo.push_back( _sourceNo );


        if(_noSources > 1)
        {

          _noToFitTxt->setEnabled( true );
          _noToFitSpinBox->setEnabled( true );
  
 



       }
 



    }
    

    fixedBackgroundHasChanged( _fixedBackground );


    updateDisplay();


}
/****************************************************************
*
*
****************************************************************/

void FastPhotometryDlg::fixedBackgroundHasChanged( bool type)
{

    _fixedBackground = type; 
    

    if( type )   // Using a fixed background
    {

        _computeBackgroundBtn->setEnabled( true );
        _inputBackgroundBtn->setEnabled( true );

        _backgroundTxt->setEnabled( true );   
        _backgroundEdt->setEnabled( true );

  
        _innerBckSpinBox->setEnabled ( false );
        _outerBckSpinBox->setEnabled ( false );


        _innerBckTxt->setEnabled ( false );
        _outerBckTxt->setEnabled ( false );


    }

    else // Using an annulus to get the background
    {

        _computeBackgroundBtn->setEnabled( false );
        _inputBackgroundBtn->setEnabled( false );

        _backgroundTxt->setEnabled( false );
        _backgroundEdt->setEnabled( false );


        if(_photometryType == 0) // Aperture photometry
	{

            _innerBckSpinBox->setEnabled ( true );
            _outerBckSpinBox->setEnabled ( true );

            _innerBckTxt->setEnabled ( true );
            _outerBckTxt->setEnabled ( true );
        } 
        else
	{

            _innerBckSpinBox->setEnabled ( false );
            _outerBckSpinBox->setEnabled ( false);

            _innerBckTxt->setEnabled ( false );
            _outerBckTxt->setEnabled ( false );
        } 
   



    }



    _computeBtn->setEnabled(true);


}


/*********************************************************
*
*********************************************************/
void FastPhotometryDlg::getBackgroundValue()
{

    /*****************************************************
    * Background subtraction using the user-supplied value
    *****************************************************/
 
       
    bool ok;
    float value = _backgroundMode; 
    float minValue =  max(0.0, min(_backgroundMode - 5.0, _backgroundMode * 0.75));
    float maxValue =  max( _backgroundMode +  5.0, _backgroundMode * 1.75);
    QString text;
    text.sprintf("%s%6.1f%s%6.1f%s%6.1f%s%6.1f", "Please change if required-\n"
		    "The allowed range of values is ", minValue, "-", maxValue, 
                    "\nThe computed mode value is", _backgroundMode,
                    "\nThe last user value was ", _backgroundUserValue);
    double res = QInputDialog::getDouble( "Please select your background value (counts/pixel)", 
                  text, value, minValue, maxValue, 1, &ok, this );  // The 1 is the number of decimal places
      
    if ( ok ) {
 
        // user entered something and pressed OK
          
        _backgroundUserValue = res;   

        _backgroundEdt->setText(text.sprintf("%8.2f",  _backgroundUserValue  ));




	//        QString text;

//           text.sprintf("Uses your background value (%.1f counts/pixel) to subtract the background", _background);
//           QToolTip::add( _backgroundRadioBtn[2] , text);
  
  
    }




   _computeBtn->setEnabled(true);



}

/*********************************************************
*
*
*********************************************************/
void FastPhotometryDlg::numberToFitHasChanged(int no)
{

  //  cout << "Numbertofithaschanged line 2880 no=" << no << endl;


    _numberToFit = no; // The total number of sources in the least-squares PSF fit


    _fitNo.clear();

   
    _computeBtn->setEnabled(true);



    if(no == 1)
    {

        _fitNo.push_back( _sourceNo );


        return;
    }





  
    //    cout << "numbertofithaschanged no=" << no << endl;
    /***********************************************************************************
    * Compute the maximum radius from thye main source to neighbouring ones
    * that will be included in the fit
    ************************************************************************************/ 
    PhotometryParameters photParameters = F90MODVAR(omsource_glue, myphotometryparameters);
    float maxRadius = float(24 / photParameters.binX);


 
    vector<OmSourceParameter> sourceList;


    OmSourceParameters& sources = F90MODVAR( omsource_glue, mysources );   
 
 
 



    /************************************************************
    *
    * Obtain a list of sources within a maximum distance maxRadius 
    * of the photometry source
    *
    *************************************************************/

    //    cout << "Fitting to source " << _sourceNo << " source.number=" << _source.number 
    //     << " x=" << _source.xcoord << " y=" << _source.ycoord << endl;

    for (int i = sources.lbound(); i <= sources.ubound(); i++) 
    {

        
        double dx = _source.xcoord - sources[i].xcoord;
        double dy = _source.ycoord - sources[i].ycoord;
        float radius = hypot(dx, dy);
        
        if(radius <= maxRadius)
	{
	    sources[i].alpha = radius; 
            sources[i].number = i;
	    sourceList.push_back( sources[i] );
	    //            cout << i << ") Line 2947 adding source " 
	    //		 << " number=" << sources[i].number << " x=" << 
            //         sources[i].xcoord << " y=" << sources[i].ycoord << " to list of sources for psf fit\n";
              
	    if( (int)sourceList.size() == 0) 
	    {

	        break;  // Got all the sources for the fit 
            }            
        }


    }

    /************************************************************************
    *
    * Sort into increasing distance order
    *
    ************************************************************************/

    //    cout << "omfastphotometrydlg.cc line 2937 Before sorting sourcelist.number\n";

    //  for(int i = 0; i < no; i++)
    // {
    //	cout << i << ") sourceList[i].number=" << sourceList[i].number << endl;
    //  }


    sort( sourceList.begin(), sourceList.end(), compare);

    // cout << "numberofithaschanged line 2984 no=" << no << endl;

    // cout << "After sorting list of sources for fit=\n";
    for(int i = 0; i < no; i++)
    {
        _fitNo.push_back( sourceList[i].number );  //  Record the number of the source

	//	cout << i << ") sourceList[i].number=" << sourceList[i].number << endl;

    }


  

}
/**********************************************************
*
**********************************************************/
void FastPhotometryDlg::weightingHasChanged(bool set)
{
    int method = 0;

    if(set) 
      method = 1;
    else
      method = 2;

    F90MODPROC(omsource_old, setweightingmethod)( &method );
 
    _computeBtn->setEnabled(true);

}
/****************************************************
*
***************************************************/
void FastPhotometryDlg::updateCputime()
{

    QString text;
    text.sprintf("Execution time=%d seconds", _time.elapsed() / 1000);
          
    _instructionsLabel->setText(text, 20);

    qApp->processEvents();


     
}
